@@ -329,9 +329,18 @@ class DataBentoDataBacktesting(PandasData):
                 if not df.empty and 'close' in df.columns:
                         # Ensure current_dt is timezone-aware for comparison
                         current_dt_aware = to_datetime_aware(current_dt)
-                        
-                        # Filter to data up to current backtest time
-                        filtered_df = df[df.index <= current_dt_aware]
+
+                        # Step back one bar so only fully closed bars are visible
+                        bar_delta = timedelta(minutes=1)
+                        if asset_data.timestep == "hour":
+                            bar_delta = timedelta(hours=1)
+                        elif asset_data.timestep == "day":
+                            bar_delta = timedelta(days=1)
+
+                        cutoff_dt = current_dt_aware - bar_delta
+
+                        # Filter to data up to current backtest time (exclude current bar unless broker overrides)
+                        filtered_df = df[df.index <= cutoff_dt]
                         
                         if not filtered_df.empty:
                             last_price = filtered_df['close'].iloc[-1]
@@ -410,14 +419,23 @@ class DataBentoDataBacktesting(PandasData):
                     if not df.empty:
                         # Apply timeshift if specified
                         current_dt = self.get_datetime()
+                        shift_seconds = 0
                         if timeshift:
-                            current_dt = current_dt - timeshift
+                            if isinstance(timeshift, int):
+                                shift_seconds = timeshift * 60
+                                current_dt = current_dt - timedelta(minutes=timeshift)
+                            else:
+                                shift_seconds = timeshift.total_seconds()
+                                current_dt = current_dt - timeshift
                         
                         # Ensure current_dt is timezone-aware for comparison
                         current_dt_aware = to_datetime_aware(current_dt)
                         
-                        # Filter data up to current backtest time
-                        filtered_df = df[df.index <= current_dt_aware]
+                        # Filter data up to current backtest time (exclude current bar unless broker overrides)
+                        include_current = getattr(self, "_include_current_bar_for_orders", False)
+                        allow_current = include_current or shift_seconds > 0
+                        mask = df.index <= current_dt_aware if allow_current else df.index < current_dt_aware
+                        filtered_df = df[mask]
                         
                         # Take the last 'length' bars
                         result_df = filtered_df.tail(length)
@@ -480,14 +498,29 @@ class DataBentoDataBacktesting(PandasData):
             if not df.empty:
                 # Apply timeshift if specified
                 current_dt = self.get_datetime()
+                shift_seconds = 0
                 if timeshift:
-                    current_dt = current_dt - timedelta(minutes=timeshift)
+                    if isinstance(timeshift, int):
+                        shift_seconds = timeshift * 60
+                        current_dt = current_dt - timedelta(minutes=timeshift)
+                    else:
+                        shift_seconds = timeshift.total_seconds()
+                        current_dt = current_dt - timeshift
                 
                 # Ensure current_dt is timezone-aware for comparison
                 current_dt_aware = to_datetime_aware(current_dt)
                 
-                # Filter data up to current backtest time
-                filtered_df = df[df.index <= current_dt_aware]
+                # Step back one bar to avoid exposing the in-progress bar
+                bar_delta = timedelta(minutes=1)
+                if asset_data.timestep == "hour":
+                    bar_delta = timedelta(hours=1)
+                elif asset_data.timestep == "day":
+                    bar_delta = timedelta(days=1)
+
+                cutoff_dt = current_dt_aware - bar_delta
+
+                # Filter data up to current backtest time (exclude current bar unless broker overrides)
+                filtered_df = df[df.index <= cutoff_dt] if shift_seconds > 0 else df[df.index < current_dt_aware]
                 
                 # Take the last 'length' bars
                 result_df = filtered_df.tail(length)
@@ -531,4 +564,4 @@ class DataBentoDataBacktesting(PandasData):
         # Prefetch data for all assets
         self.prefetch_data(assets, timestep)
         
-        logger.info(f"Initialized DataBento backtesting with prefetched data for {len(assets)} assets")
\ No newline at end of file
+        logger.info(f"Initialized DataBento backtesting with prefetched data for {len(assets)} assets")
