@@ -157,12 +157,8 @@ else:
 POLYGON_MAX_MEMORY_BYTES = os.environ.get("POLYGON_MAX_MEMORY_BYTES")
 
 POLYGON_CONFIG = {
-    # Add POLYGON_API_KEY and POLYGON_IS_PAID_SUBSCRIPTION to your .env file or set them as secrets
+    # Add POLYGON_API_KEY to your .env file or set it as secrets
     "API_KEY": os.environ.get("POLYGON_API_KEY"),
-    "IS_PAID_SUBSCRIPTION": os.environ.get("POLYGON_IS_PAID_SUBSCRIPTION").lower()
-    == "true"
-    if os.environ.get("POLYGON_IS_PAID_SUBSCRIPTION")
-    else False,
 }
 
 # Polygon API Key
@@ -513,13 +509,15 @@ if not is_backtesting or is_backtesting.lower() == "false":
                 # Handle rate limiting and other connection errors gracefully
                 error_str = str(e)
                 if "rate limited" in error_str.lower() or "429" in error_str:
-                    logger.warning(
+                    message = (
                         "Tradovate connection blocked due to rate limiting. "
                         "Too many requests were made. Wait 5-10 minutes and try again."
                     )
+                    logger.error(termcolor.colored(message, "red"))
+                    raise RuntimeError(message) from e
                 else:
-                    logger.warning(f"Could not initialize Tradovate broker: {e}")
-                broker = None
+                    logger.error(termcolor.colored(f"Could not initialize Tradovate broker: {e}", "red"))
+                    raise
         # Only check for SCHWAB_ACCOUNT_NUMBER to select Schwab
         elif SCHWAB_CONFIG.get("SCHWAB_ACCOUNT_NUMBER"):
             broker = Schwab(SCHWAB_CONFIG)
