@@ -45,6 +45,7 @@ class PolygonDataPolars(PolarsMixin, DataSourceBacktesting):
         {"timestep": "minute", "representations": ["1m", "1 minute", "minute"]},
         {"timestep": "hour", "representations": ["1h", "1 hour", "hour"]},
     ]
+    option_quote_fallback_allowed = True
 
     def __init__(
         self,
@@ -283,19 +284,23 @@ class PolygonDataPolars(PolarsMixin, DataSourceBacktesting):
             # Assuming e.message or similar attribute contains the error message
             formatted_start_datetime = start_datetime.strftime("%Y-%m-%d")
             formatted_end_datetime = self.datetime_end.strftime("%Y-%m-%d")
-            if "Your plan doesn't include this data timeframe" in str(e):
-                error_message = colored(
+            text = str(e)
+            plan_msgs = (
+                "Your plan doesn't include this data timeframe",
+                "Your plan doesn\u2019t include this data timeframe",
+                "not entitled to this data",
+                "NOT_AUTHORIZED",
+            )
+            invalid_key_msgs = ("Unknown API Key", "Invalid API Key")
+            if any(m in text for m in plan_msgs) and not any(m in text for m in invalid_key_msgs):
+                msg = (
                     "Polygon Access Denied: Your subscription does not allow you to backtest that far back in time. "
-                    f"You requested data for {asset_separated} {ts_unit} bars "
-                    f"from {formatted_start_datetime} to {formatted_end_datetime}. "
-                    "Please consider either changing your backtesting timeframe to start later since your "
-                    "subscription does not allow you to backtest that far back or upgrade your Polygon "
-                    "subscription."
-                    "You can upgrade your Polygon subscription at at https://polygon.io/?utm_source=affiliate&utm_campaign=lumi10 "
-                    "Please use the full link to give us credit for the sale, it helps support this project. "
-                    "You can use the coupon code 'LUMI10' for 10% off. ",
-                    color="red")
-                raise Exception(error_message) from e
+                    f"Requested {asset_separated} {ts_unit} bars from {formatted_start_datetime} to {formatted_end_datetime}. "
+                    "Consider starting later or upgrading your Polygon subscription (https://polygon.io/?utm_source=affiliate&utm_campaign=lumi10, code 'LUMI10')."
+                )
+                logger.error(colored(msg, color="red"))
+                # Non-fatal: skip this download window and continue
+                return
             elif "Unknown API Key" in str(e):
                 error_message = colored(
                     "Polygon Access Denied: Your API key is invalid. "
@@ -563,9 +568,11 @@ class PolygonDataPolars(PolarsMixin, DataSourceBacktesting):
         if start_date or end_date:
             filters = []
             if start_date:
-                filters.append(pl.col("datetime") >= start_date)
+                # Use pl.lit() to ensure datetime precision compatibility across Polars versions
+                filters.append(pl.col("datetime") >= pl.lit(start_date))
             if end_date:
-                filters.append(pl.col("datetime") <= end_date)
+                # Use pl.lit() to ensure datetime precision compatibility across Polars versions
+                filters.append(pl.col("datetime") <= pl.lit(end_date))
 
             response = lazy_data.filter(pl.all_horizontal(filters)).collect()
         else:
