@@ -1,10 +1,13 @@
-import logging
 from decimal import Decimal
 from typing import Union
 
 from termcolor import colored
-from lumibot.entities import Asset, Bars
+
 from lumibot.data_sources import DataSource
+from lumibot.entities import Asset, Bars
+from lumibot.tools.lumibot_logger import get_logger
+
+logger = get_logger(__name__)
 
 class TradovateData(DataSource):
     """
@@ -51,7 +54,7 @@ class TradovateData(DataSource):
         return headers
 
     def get_chains(self, asset: Asset, quote: Asset = None) -> dict:
-        logging.error(colored("Method 'get_chains' does not work with Tradovate.", "red"))
+        logger.error(colored("Method 'get_chains' does not work with Tradovate.", "red"))
         return {}
 
     def get_historical_prices(
@@ -65,7 +68,7 @@ class TradovateData(DataSource):
         """
 
         # Log that this method is not supported because Tradovate requires you to get a CME subscription which costs $440/month
-        logging.error(colored("Method 'get_historical_prices' is not implemented for Tradovate because it requires a CME subscription which costs $440/month.", "red"))
+        logger.error(colored("Method 'get_historical_prices' is not implemented for Tradovate because it requires a CME subscription which costs $440/month.", "red"))
         return None
 
     def get_last_price(self, asset, quote=None, exchange=None) -> Union[float, Decimal, None]:
@@ -74,7 +77,7 @@ class TradovateData(DataSource):
         This method first retrieves the contract ID for the asset's symbol, then subscribes
         to market data using that contract ID.
         """
-       
+
         # Log that this method is not supported because Tradovate requires you to get a CME subscription which costs $440/month
-        logging.error(colored("Method 'get_last_price' is not implemented for Tradovate because it requires a CME subscription which costs $440/month.", "red"))
-        return None
\ No newline at end of file
+        logger.error(colored("Method 'get_last_price' is not implemented for Tradovate because it requires a CME subscription which costs $440/month.", "red"))
+        return None
