@@ -17,6 +17,7 @@ from termcolor import colored
 
 from lumibot.constants import LUMIBOT_DEFAULT_PYTZ
 from lumibot.entities import Asset, Order
+from lumibot.entities import Asset
 from lumibot.tools import append_locals, get_trading_days, staticdecorator
 
 
@@ -188,7 +189,32 @@ class StrategyExecutor(Thread):
         while held_trades_len > 0:
             # Snapshot for the broker and lumibot:
             self.strategy
-            broker_balances = self.broker._get_balances_at_broker(self.strategy.quote_asset, self.strategy)
+            try:
+                broker_balances = self.broker._get_balances_at_broker(self.strategy.quote_asset, self.strategy)
+            except Exception as balance_exc:
+                # Gracefully handle rate-limit style failures by falling back to cached values when available
+                status_code = getattr(balance_exc, "status_code", None)
+                message = str(balance_exc)
+                cached_balances = getattr(self.broker, "_cached_balances", None)
+
+                if status_code == 429 or "rate limit" in message.lower():
+                    if cached_balances is not None:
+                        self.strategy.logger.warning(
+                            "Broker balance refresh hit rate limit; using cached values and continuing"
+                        )
+                        broker_balances = cached_balances
+                    else:
+                        self.strategy.logger.warning(
+                            "Broker balance refresh hit rate limit and no cached value is available; retrying"
+                        )
+                        broker_balances = None
+                else:
+                    # Unexpected failure follows legacy retry path
+                    self.strategy.logger.warning(
+                        f"Broker balance refresh failed with {balance_exc}; retrying"
+                    )
+                    broker_balances = None
+
             if broker_balances is None:
                 if cash_broker_retries < cash_broker_max_retries:
                     self.strategy.logger.info("Unable to get cash from broker, trying again.")
@@ -420,9 +446,6 @@ class StrategyExecutor(Thread):
             self._on_canceled_order(**payload)
 
         elif event == self.FILLED_ORDER:
-            # Log that we are processing a filled order.
-            self.strategy.logger.debug(f"Processing a filled order, payload: {payload}")
-
             order = payload["order"]
             price = payload["price"]
             quantity = payload["quantity"]
@@ -430,21 +453,66 @@ class StrategyExecutor(Thread):
 
             # Parent orders to not affect cash or trades directly, the individual child_orders will when they
             # are filled. Skip the parent order so as not to double count.
-            if not order.is_parent() and order.asset.asset_type != "crypto":
+            update_cash = True
+            order_class_value = getattr(order, "order_class", None)
+            try:
+                order_class_enum = (
+                    Order.OrderClass(order_class_value)
+                    if order_class_value is not None
+                    else None
+                )
+            except ValueError:
+                order_class_enum = None
+
+            if order.is_parent() and order_class_enum not in (
+                Order.OrderClass.BRACKET,
+                Order.OrderClass.OTO,
+            ):
+                update_cash = False
+
+            asset_type = getattr(order.asset, "asset_type", None)
+
+            if (
+                update_cash
+                and asset_type != Asset.AssetType.CRYPTO
+                and quantity is not None
+                and price is not None
+            ):
                 self.strategy._update_cash(order.side, quantity, price, multiplier)
 
             self._on_filled_order(**payload)
 
         elif event == self.PARTIALLY_FILLED_ORDER:
-            # Log that we are processing a partially filled order.
-            self.strategy.logger.debug(f"Processing a partially filled order, payload: {payload}")
-
             order = payload["order"]
             price = payload["price"]
             quantity = payload["quantity"]
             multiplier = payload["multiplier"]
 
-            if order.asset.asset_type != "crypto":
+            update_cash = True
+            order_class_value = getattr(order, "order_class", None)
+            try:
+                order_class_enum = (
+                    Order.OrderClass(order_class_value)
+                    if order_class_value is not None
+                    else None
+                )
+            except ValueError:
+                order_class_enum = None
+
+            if order.is_parent() and order_class_enum not in (
+                Order.OrderClass.BRACKET,
+                Order.OrderClass.OTO,
+            ):
+                update_cash = False
+
+            asset_type = getattr(order.asset, "asset_type", None)
+
+            if (
+                update_cash
+                and asset_type != Asset.AssetType.CRYPTO
+                and quantity is not None
+                and price is not None
+            ):
                 self.strategy._update_cash(order.side, quantity, price, multiplier)
 
             self._on_partially_filled_order(**payload)
@@ -1110,14 +1178,28 @@ class StrategyExecutor(Thread):
                 # Sleep until the market closes.
                 self.safe_sleep(time_to_close)
 
-                # Remove the time to close from the strategy sleep time.
-                strategy_sleeptime -= time_to_close
-
                 # Check if the broker has a function to process expired option contracts.
                 if hasattr(self.broker, "process_expired_option_contracts"):
                     # Process expired option contracts.
                     self.broker.process_expired_option_contracts(self.strategy)
 
+                # For backtesting with non-continuous markets, after reaching market close,
+                # we should end the trading session for this day and return False to break out
+                # of the backtesting loop. The main loop will then call _advance_to_next_trading_day()
+                # to move to the next trading day.
+                #
+                # IMPORTANT: Skip this ONLY for pure PandasDataBacktesting sources (not Polygon
+                # which inherits from PandasData) to maintain backward compatibility with existing
+                # tests that expect pandas daily data to process multiple days in a single call.
+                is_pure_pandas_data = (hasattr(self.broker, 'data_source') and
+                                      type(self.broker.data_source).__name__ in ('PandasData', 'PandasDataBacktesting'))
+
+                if self.strategy.is_backtesting and not is_pure_pandas_data:
+                    return False
+
+                # For live trading or pandas data, continue with the remaining sleep time
+                strategy_sleeptime -= time_to_close
+
         # TODO: next line speed implication: medium (371 microseconds)
         self.safe_sleep(strategy_sleeptime)
 
@@ -1284,6 +1366,10 @@ class StrategyExecutor(Thread):
             if not sleep_result:
                 break
 
+            # Recalculate time_to_close for the next iteration
+            if not is_continuous_market:
+                time_to_close = self.broker.get_time_to_close()
+
         # Don't log this to avoid creating root handler
         # self.strategy.log_message(f"Backtesting loop completed with {iteration_count} iterations")
 
@@ -1339,22 +1425,39 @@ class StrategyExecutor(Thread):
                 self.strategy.log_message(f"Strategy will check in again at: {dt_str}", color="blue")
 
             # Loop until the strategy should stop.
+            loop_count = 0
             while True:
+                loop_count += 1
+
+                # Log every 60 iterations (roughly every minute) to track loop activity
+                if loop_count % 60 == 1:
+                    self.strategy.logger.debug(f"Main loop iteration #{loop_count} - Market closed status check")
+
+                # Send data to cloud every minute FIRST - regardless of market status
+                should_send_cloud_update = (not hasattr(self, '_last_updated_cloud')) or ((datetime.now() - self._last_updated_cloud) >= timedelta(minutes=1))
+                if should_send_cloud_update:
+                    time_since_last = "never" if not hasattr(self, '_last_updated_cloud') else str(datetime.now() - self._last_updated_cloud)
+                    self.strategy.logger.debug(f"Sending cloud update (last update: {time_since_last} ago)")
+                    self.strategy.send_update_to_cloud()
+                    self._last_updated_cloud = datetime.now()
+
                 # Get the current jobs from the scheduler (may be None if gracefully exited previously)
                 if self.scheduler is None:
+                    self.strategy.log_message("⚠️ Scheduler is None, attempting to recreate", color="yellow")
                     # Attempt to re-create and start the scheduler
                     self._setup_live_trading_scheduler()
 
                 jobs = self.scheduler.get_jobs() if self.scheduler is not None else []
 
-                # Check if we should continue trading loop
-                if not self._should_continue_trading_loop(jobs, is_continuous_market, should_we_stop):
-                    break
+                # Log scheduler status every minute
+                if loop_count % 60 == 1:
+                    self.strategy.logger.debug(f"Scheduler jobs: {len(jobs)} active")
 
-                # Send data to cloud every minute. Ensure not being in a trading iteration currently as it can cause an incomplete data sync
-                if (not hasattr(self, '_last_updated_cloud')) or ((datetime.now() - self._last_updated_cloud) >= timedelta(minutes=1)):
-                    self.strategy.send_update_to_cloud()
-                    self._last_updated_cloud = datetime.now()
+                # Check if we should continue trading loop
+                should_continue = self._should_continue_trading_loop(jobs, is_continuous_market, should_we_stop)
+                if not should_continue:
+                    self.strategy.logger.debug(f"Trading loop should stop: jobs={len(jobs)}, continuous={is_continuous_market}, should_stop={should_we_stop}")
+                    break
 
                 # Handle LifeCycle methods
                 self._handle_lifecycle_methods()
@@ -1457,8 +1560,19 @@ class StrategyExecutor(Thread):
             return True
 
         except Exception as e:
-            # Store the exception so the main thread can check it
-            self.exception = e
-            self.result = self.strategy._analysis if hasattr(self.strategy, '_analysis') else {}
-            # Don't re-raise - let the main thread handle it
+            # Log and surface any exceptions that occur before/around initialize so they are never silent
+            try:
+                self.strategy.logger.error(e)
+                self.strategy.logger.error(traceback.format_exc())
+                # Attempt to notify the strategy via on_bot_crash hook
+                try:
+                    self._on_bot_crash(e)
+                except Exception as e1:
+                    self.strategy.logger.error(e1)
+                    self.strategy.logger.error(traceback.format_exc())
+            finally:
+                # Store the exception so the main thread can check it
+                self.exception = e
+                self.result = self.strategy._analysis if hasattr(self.strategy, '_analysis') else {}
+            # Don't re-raise here; main thread (Trader) will handle raising/logging
             return False
