@@ -1,5 +1,6 @@
 import logging
 import queue
+import threading
 from queue import Queue
 
 class CustomStream:
@@ -7,13 +8,24 @@ class CustomStream:
     def __init__(self):
         self._queue = Queue(100)
         self._actions_mapping = {}
+        self._stop_event = threading.Event()
+        self._thread = None
 
     def dispatch(self, event, wait_until_complete=False, **payload):
-        self._queue.put((event, payload), block=False)
+        # Don't put events if we're stopping
+        if self._stop_event.is_set():
+            return
+
+        try:
+            self._queue.put((event, payload), block=False)
+        except queue.Full:
+            logging.warning(f"Queue full, dropping event {event}")
+            return
 
         # Primarily used for backtesting. If wait_until_complete is True, the function will block until the queue is
         # empty. This is useful for ensuring that all events have been processed before moving on to the next step.
         if wait_until_complete:
+            # Wait for the queue to be processed
             self._queue.join()
 
     def add_action(self, event_name):
@@ -24,10 +36,21 @@ class CustomStream:
         return add_event_action
 
     def _run(self):
-        while True:
-            event, payload = self._queue.get()  # This is a blocking operation.
-            self._process_queue_event(event, payload)
-            self._queue.task_done()
+        while not self._stop_event.is_set():
+            try:
+                # Use timeout to periodically check stop_event
+                event, payload = self._queue.get(timeout=0.1)
+                self._process_queue_event(event, payload)
+                self._queue.task_done()
+            except queue.Empty:
+                # Check if we should stop
+                if self._stop_event.is_set():
+                    break
+                continue
+            except Exception as e:
+                logging.error(f"Error processing queue event: {e}")
+                if self._stop_event.is_set():
+                    break
 
     def _process_queue_event(self, event, payload):
         if payload is None:
@@ -40,6 +63,20 @@ class CustomStream:
         # Threads are spawned by the broker._launch_stream() code
         self._run()
 
+    def stop(self):
+        """Stop the stream gracefully"""
+        self._stop_event.set()
+        # Clear the queue to unblock any waiting operations
+        try:
+            while not self._queue.empty():
+                self._queue.get_nowait()
+                self._queue.task_done()
+        except queue.Empty:
+            pass
+        # Wait for thread to finish if it exists
+        if self._thread and self._thread.is_alive():
+            self._thread.join(timeout=1)
+
 
 class PollingStream(CustomStream):
     """
@@ -61,19 +98,25 @@ class PollingStream(CustomStream):
         self.polling_interval = polling_interval
 
     def _run(self):
-        while True:
+        while not self._stop_event.is_set():
             try:
                 # This is a blocking operation until an item is available in the queue or the timeout is reached.
-                event, payload = self._queue.get(timeout=self.polling_interval)
+                event, payload = self._queue.get(timeout=min(self.polling_interval, 0.1))
                 self._process_queue_event(event, payload)
                 self._queue.task_done()
             except queue.Empty:
-                # If the queue is empty, it means the polling interval has been reached.
-                self._poll()
+                # Check if we should stop
+                if self._stop_event.is_set():
+                    break
+                # If the queue is empty and enough time has passed, poll
+                if not self._stop_event.is_set():
+                    self._poll()
                 continue
             # Ensure that the Polling thread does not die if an exception is raised in the event processing.
             except Exception as e: # noqa
                 logging.exception(f"An error occurred while processing a queue event. {e}")
+                if self._stop_event.is_set():
+                    break
                 continue
 
     def _poll(self):
