@@ -3,7 +3,6 @@ Unit tests for continuous futures resolution in DataBento helper.
 """
 
 import unittest
-from unittest.mock import patch, MagicMock
 from datetime import datetime, date
 
 from lumibot.tools.databento_helper import (
@@ -19,51 +18,32 @@ class TestContinuousFuturesResolution(unittest.TestCase):
         """Set up test fixtures."""
         self.test_date = datetime(2025, 7, 2)  # July 2, 2025
         
-    @patch('datetime.datetime')
-    def test_generate_current_futures_contract_july(self, mock_datetime):
+    def test_generate_current_futures_contract_july(self):
         """Test contract generation for July 2025."""
-        mock_datetime.now.return_value = self.test_date
-        
-        # Create continuous futures asset and test using Asset methods
         asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        contract = asset.resolve_continuous_futures_contract()
+        contract = asset.resolve_continuous_futures_contract(reference_date=self.test_date)
         self.assertEqual(contract, 'MESU25')
-        
-        # Test with ES as well
+
         asset_es = Asset("ES", asset_type=Asset.AssetType.CONT_FUTURE)
-        contract = asset_es.resolve_continuous_futures_contract()
+        contract = asset_es.resolve_continuous_futures_contract(reference_date=self.test_date)
         self.assertEqual(contract, 'ESU25')
-    
-    @patch('datetime.datetime')
-    def test_generate_potential_contracts_july(self, mock_datetime):
+
+    def test_generate_potential_contracts_july(self):
         """Test potential contracts generation for July 2025."""
-        mock_datetime.now.return_value = self.test_date
-        
-        asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        contracts = asset.get_potential_futures_contracts()
-        
-        # Should prioritize September 2025, then December 2025, then March 2026
-        self.assertIn('MESU25', contracts)  # September 2025
-        self.assertIn('MESZ25', contracts)  # December 2025
-        self.assertIn('MESH26', contracts)  # March 2026
-        
-        # September should come before December
-        sep_index = contracts.index('MESU25')
-        dec_index = contracts.index('MESZ25')
-        self.assertLess(sep_index, dec_index)
+        contracts = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE).get_potential_futures_contracts(
+            reference_date=self.test_date
+        )
+
+        self.assertIn('MESU25', contracts)
+        self.assertIn('MESZ25', contracts)
+        self.assertIn('MESH26', contracts)
+        self.assertLess(contracts.index('MESU25'), contracts.index('MESZ25'))
     
     def test_format_continuous_futures_symbol(self):
         """Test formatting of continuous futures symbols."""
-        # Create continuous futures asset
         asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        
-        with patch('lumibot.tools.databento_helper.datetime') as mock_datetime:
-            mock_datetime.now.return_value = self.test_date
-            
-            # Should resolve to a specific contract with DataBento formatting
-            resolved = _format_futures_symbol_for_databento(asset)
-            # DataBento helper applies its own formatting: MESU25 -> MESU5
-            self.assertEqual(resolved, 'MESU5')
+        resolved = _format_futures_symbol_for_databento(asset, self.test_date)
+        self.assertEqual(resolved, 'MESU5')
     
     def test_format_specific_futures_symbol(self):
         """Test formatting of specific futures contracts."""
@@ -75,13 +55,11 @@ class TestContinuousFuturesResolution(unittest.TestCase):
         resolved = _format_futures_symbol_for_databento(asset)
         self.assertEqual(resolved, 'MESZ25')
     
-    @patch('datetime.datetime')
-    def test_contract_ordering_priority(self, mock_datetime):
+    def test_contract_ordering_priority(self):
         """Test that contracts are ordered by trading priority."""
-        mock_datetime.now.return_value = self.test_date
-        
-        asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        contracts = asset.get_potential_futures_contracts()
+        contracts = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE).get_potential_futures_contracts(
+            reference_date=self.test_date
+        )
         
         # First few contracts should be the most liquid quarterly months
         # For July, September should be first (closest quarter)
@@ -93,20 +71,18 @@ class TestContinuousFuturesResolution(unittest.TestCase):
     
     def test_month_code_mapping(self):
         """Test that month codes are correctly mapped."""
-        with patch('datetime.datetime') as mock_datetime:
-            # Test different months
-            test_cases = [
-                (datetime(2025, 1, 15), 'H'),  # Jan -> Mar (H)  
-                (datetime(2025, 4, 15), 'M'),  # Apr -> Jun (M)
-                (datetime(2025, 7, 15), 'U'),  # Jul -> Sep (U)
-                (datetime(2025, 10, 15), 'Z'), # Oct -> Dec (Z)
-            ]
-            
-            for test_date, expected_month_code in test_cases:
-                mock_datetime.now.return_value = test_date
-                asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-                contract = asset.resolve_continuous_futures_contract()
-                self.assertIn(expected_month_code, contract)
+        test_cases = [
+            (datetime(2025, 1, 15), 'H'),
+            (datetime(2025, 4, 15), 'M'),
+            (datetime(2025, 7, 15), 'U'),
+            (datetime(2025, 10, 15), 'Z'),
+        ]
+
+        for test_date, expected_month_code in test_cases:
+            contract = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE).resolve_continuous_futures_contract(
+                reference_date=test_date
+            )
+            self.assertIn(expected_month_code, contract)
 
     def test_error_handling_non_continuous_futures(self):
         """Test that proper errors are raised for non-continuous futures assets."""
@@ -127,50 +103,34 @@ class TestContinuousFuturesResolution(unittest.TestCase):
             stock_asset.get_potential_futures_contracts()
         self.assertIn("can only be called on CONT_FUTURE assets", str(context.exception))
 
-    @patch('datetime.datetime')
-    def test_year_rollover_edge_cases(self, mock_datetime):
+    def test_year_rollover_edge_cases(self):
         """Test contract generation around year boundaries with expiration-aware logic."""
-        # Test December rollover - after Dec 15th, should use March next year
-        # (December contract expires ~Dec 19th, so Dec 31st would be using expired contract)
-        mock_datetime.now.return_value = datetime(2025, 12, 31)
         asset = Asset("ES", asset_type=Asset.AssetType.CONT_FUTURE)
-        contract = asset.resolve_continuous_futures_contract()
-        self.assertEqual(contract, 'ESH26')  # March 2026 (after rollover)
-        
-        # Test January after rollover - should still be March
-        mock_datetime.now.return_value = datetime(2026, 1, 1)
-        contract = asset.resolve_continuous_futures_contract()
-        self.assertEqual(contract, 'ESH26')  # March 2026
-        
-        # Test before December rollover - should still use December
-        mock_datetime.now.return_value = datetime(2025, 12, 14)
-        contract = asset.resolve_continuous_futures_contract()
-        self.assertEqual(contract, 'ESZ25')  # December 2025 (before rollover)
 
-    @patch('datetime.datetime')
-    def test_different_symbol_formats(self, mock_datetime):
+        contract = asset.resolve_continuous_futures_contract(reference_date=datetime(2025, 12, 31))
+        self.assertEqual(contract, 'ESH26')
+
+        contract = asset.resolve_continuous_futures_contract(reference_date=datetime(2026, 1, 1))
+        self.assertEqual(contract, 'ESH26')
+
+        contract = asset.resolve_continuous_futures_contract(reference_date=datetime(2025, 12, 14))
+        self.assertEqual(contract, 'ESZ25')
+
+    def test_different_symbol_formats(self):
         """Test continuous futures resolution with different symbol formats."""
-        mock_datetime.now.return_value = self.test_date
-        
-        # Test various symbol formats
         test_symbols = ["MES", "ES", "NQ", "RTY", "CL", "GC", "SI"]
-        
-        for symbol in test_symbols:
-            asset = Asset(symbol, asset_type=Asset.AssetType.CONT_FUTURE)
-            contract = asset.resolve_continuous_futures_contract()
-            
-            # Should start with the symbol
-            self.assertTrue(contract.startswith(symbol))
-            # Should have month code and year
-            self.assertTrue(len(contract) >= len(symbol) + 3)  # symbol + month + 2-digit year
 
-    @patch('datetime.datetime')
-    def test_potential_contracts_uniqueness(self, mock_datetime):
+        for symbol in test_symbols:
+            contract = Asset(symbol, asset_type=Asset.AssetType.CONT_FUTURE).resolve_continuous_futures_contract(
+                reference_date=self.test_date
+            )
+            self.assertTrue(contract.startswith(symbol))
+            self.assertTrue(len(contract) >= len(symbol) + 3)
+
+    def test_potential_contracts_uniqueness(self):
         """Test that potential contracts list contains no duplicates."""
-        mock_datetime.now.return_value = self.test_date
-        
         asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        contracts = asset.get_potential_futures_contracts()
+        contracts = asset.get_potential_futures_contracts(reference_date=self.test_date)
         
         # Check for uniqueness
         self.assertEqual(len(contracts), len(set(contracts)))
@@ -184,13 +144,10 @@ class TestContinuousFuturesResolution(unittest.TestCase):
         self.assertGreater(len(dot_formats), 0) 
         self.assertGreater(len(full_year_formats), 0)
 
-    @patch('datetime.datetime')
-    def test_quarterly_contract_preference(self, mock_datetime):
+    def test_quarterly_contract_preference(self):
         """Test that quarterly contracts (Mar, Jun, Sep, Dec) are preferred."""
-        mock_datetime.now.return_value = self.test_date
-        
         asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        contracts = asset.get_potential_futures_contracts()
+        contracts = asset.get_potential_futures_contracts(reference_date=self.test_date)
         
         # Extract month codes from contracts
         quarterly_months = {'H', 'M', 'U', 'Z'}  # Mar, Jun, Sep, Dec
@@ -209,8 +166,7 @@ class TestContinuousFuturesResolution(unittest.TestCase):
         # At least half should be quarterly contracts
         self.assertGreaterEqual(quarterly_count, 3)
 
-    @patch('datetime.datetime')
-    def test_month_progression_logic(self, mock_datetime):
+    def test_month_progression_logic(self):
         """Test that month selection follows proper logic with expiration-aware rollover."""
         asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
         
@@ -233,56 +189,39 @@ class TestContinuousFuturesResolution(unittest.TestCase):
         ]
         
         for month, expected_month_code in test_cases:
-            mock_datetime.now.return_value = datetime(2025, month, 15)
-            contract = asset.resolve_continuous_futures_contract()
+            contract = asset.resolve_continuous_futures_contract(
+                reference_date=datetime(2025, month, 15)
+            )
             actual_month_code = contract[-3]  # Third character from end
             self.assertEqual(actual_month_code, expected_month_code, 
                            f"Month {month} (15th) should resolve to {expected_month_code}, got {actual_month_code}")
 
     def test_contract_format_validation(self):
         """Test that generated contracts follow proper format conventions."""
-        with patch('datetime.datetime') as mock_datetime:
-            mock_datetime.now.return_value = self.test_date
-            
-            asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-            
-            # Test single contract format
-            contract = asset.resolve_continuous_futures_contract()
-            self.assertRegex(contract, r'^[A-Z]+[A-Z]\d{2}$')  # Format: SYMBOLMONTHYY
-            
-            # Test potential contracts formats
-            contracts = asset.get_potential_futures_contracts()
-            
-            for contract in contracts:
-                # Should match one of the expected formats
-                is_valid_format = (
-                    # Standard format: MESZ25
-                    contract.count('.') == 0 and contract[-3:].isalnum() or
-                    # Dot format: MES.Z25  
-                    contract.count('.') == 1 and contract.split('.')[1].isalnum() or
-                    # Full year format: MESZ2025
-                    contract.count('.') == 0 and len(contract) > 6
-                )
-                self.assertTrue(is_valid_format, f"Invalid contract format: {contract}")
+        asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
 
-    @patch('datetime.datetime')
-    def test_integration_with_databento_helper(self, mock_datetime):
+        contract = asset.resolve_continuous_futures_contract(reference_date=self.test_date)
+        self.assertRegex(contract, r'^[A-Z]+[A-Z]\d{2}$')
+
+        contracts = asset.get_potential_futures_contracts(reference_date=self.test_date)
+        for contract in contracts:
+            is_valid_format = (
+                ('.' not in contract and contract[-3:].isalnum()) or
+                ('.' in contract and contract.split('.')[1].isalnum()) or
+                ('.' not in contract and contract[-4:].isdigit())
+            )
+            self.assertTrue(is_valid_format, f"Invalid contract format: {contract}")
+
+    def test_integration_with_databento_helper(self):
         """Test integration between Asset class and DataBento helper."""
-        mock_datetime.now.return_value = self.test_date
-        
-        # Test that DataBento helper uses Asset class methods
         continuous_asset = Asset("MES", asset_type=Asset.AssetType.CONT_FUTURE)
-        
-        # This should use Asset.resolve_continuous_futures_contract()
-        resolved_symbol = _format_futures_symbol_for_databento(continuous_asset)
-        expected_symbol = continuous_asset.resolve_continuous_futures_contract()
-        
-        # Asset class returns full format, DataBento helper applies its formatting
-        self.assertEqual(expected_symbol, "MESU25")  # Asset class format
-        self.assertEqual(resolved_symbol, "MESU5")   # DataBento format
+        resolved_symbol = _format_futures_symbol_for_databento(continuous_asset, self.test_date)
+        expected_symbol = continuous_asset.resolve_continuous_futures_contract(reference_date=self.test_date)
 
-    @patch('datetime.datetime')
-    def test_quarterly_contract_consistency(self, mock_datetime):
+        self.assertEqual(expected_symbol, "MESU25")
+        self.assertEqual(resolved_symbol, "MESU5")
+
+    def test_quarterly_contract_consistency(self):
         """
         Test that contract resolution follows quarterly pattern with proper expiration handling.
         This test prevents regression of the original MES backtesting issue while ensuring
@@ -316,14 +255,15 @@ class TestContinuousFuturesResolution(unittest.TestCase):
         ]
         
         for test_date, expected_suffix in quarterly_tests:
-            mock_datetime.now.return_value = test_date
-            contract = asset.resolve_continuous_futures_contract()
+            contract = asset.resolve_continuous_futures_contract(reference_date=test_date)
             expected_contract = f"MES{expected_suffix}"
-            self.assertEqual(contract, expected_contract,
-                           f"Date {test_date.strftime('%Y-%m-%d')} should resolve to {expected_contract}, got {contract}")
+            self.assertEqual(
+                contract,
+                expected_contract,
+                f"Date {test_date.strftime('%Y-%m-%d')} should resolve to {expected_contract}, got {contract}",
+            )
 
-    @patch('datetime.datetime')
-    def test_expiration_edge_cases(self, mock_datetime):
+    def test_expiration_edge_cases(self):
         """
         Test contract resolution around actual futures expiration dates.
         This ensures we properly avoid expired contracts.
@@ -349,10 +289,12 @@ class TestContinuousFuturesResolution(unittest.TestCase):
         ]
         
         for test_date, expected_contract in test_cases:
-            mock_datetime.now.return_value = test_date
-            contract = asset.resolve_continuous_futures_contract()
-            self.assertEqual(contract, expected_contract,
-                           f"Date {test_date.strftime('%Y-%m-%d')} should resolve to {expected_contract}, got {contract}")
+            contract = asset.resolve_continuous_futures_contract(reference_date=test_date)
+            self.assertEqual(
+                contract,
+                expected_contract,
+                f"Date {test_date.strftime('%Y-%m-%d')} should resolve to {expected_contract}, got {contract}",
+            )
 
 
 if __name__ == '__main__':
